/*
Package auth provides common functionality for Terraform and Terraform
plugins to authenticate with OpenStack.

It includes a wide array of features, such as flexible authentication,
debugging, client creation, endpoint customization and more.

While this package is specific to Terraform, it can be used for any kind
of Terraform feature: core, backend, and plugins.
*/
package auth
